@@ -23,68 +23,56 @@
  ***************************************************************************/
 #include "tool_setup.h"
 
-#include <curlx.h>
-
 #include "tool_cfgable.h"
 #include "tool_msgs.h"
 #include "tool_cb_prg.h"
 #include "terminal.h"
 
-#include <memdebug.h> /* keep this as LAST include */
-
-#define WARN_PREFIX "Warning: "
-#define NOTE_PREFIX "Note: "
+#define WARN_PREFIX  "Warning: "
+#define NOTE_PREFIX  "Note: "
 #define ERROR_PREFIX "curl: "
 
-static void voutf(struct GlobalConfig *config,
-                  const char *prefix,
+static void voutf(const char *prefix,
                   const char *fmt,
-                  va_list ap) CURL_PRINTF(3, 0);
+                  va_list ap) CURL_PRINTF(2, 0);
 
-static void voutf(struct GlobalConfig *config,
-                  const char *prefix,
+static void voutf(const char *prefix,
                   const char *fmt,
                   va_list ap)
 {
-  size_t width = (get_terminal_columns() - strlen(prefix));
+  size_t len;
+  char *ptr;
+  char buffer[1024];
+  size_t termw = get_terminal_columns();
+  size_t prefw = strlen(prefix);
+  size_t width = termw > prefw ? termw - prefw : SIZE_MAX;
   DEBUGASSERT(!strchr(fmt, '\n'));
-  if(!config->silent) {
-    size_t len;
-    char *ptr;
-    char *print_buffer;
+  len = curl_mvsnprintf(buffer, sizeof(buffer), fmt, ap);
+  ptr = buffer;
+  while(len > 0) {
+    fputs(prefix, tool_stderr);
 
-    print_buffer = vaprintf(fmt, ap);
-    if(!print_buffer)
-      return;
-    len = strlen(print_buffer);
+    if(len > width) {
+      size_t cut = width - 1;
 
-    ptr = print_buffer;
-    while(len > 0) {
-      fputs(prefix, tool_stderr);
-
-      if(len > width) {
-        size_t cut = width-1;
-
-        while(!ISBLANK(ptr[cut]) && cut) {
-          cut--;
-        }
-        if(0 == cut)
-          /* not a single cutting position was found, just cut it at the
-             max text width then! */
-          cut = width-1;
-
-        (void)fwrite(ptr, cut + 1, 1, tool_stderr);
-        fputs("\n", tool_stderr);
-        ptr += cut + 1; /* skip the space too */
-        len -= cut + 1;
-      }
-      else {
-        fputs(ptr, tool_stderr);
-        fputs("\n", tool_stderr);
-        len = 0;
+      while(!ISBLANK(ptr[cut]) && cut) {
+        cut--;
       }
+      if(cut == 0)
+        /* not a single cutting position was found, just cut it at the
+           max text width then! */
+        cut = width - 1;
+
+      (void)fwrite(ptr, cut + 1, 1, tool_stderr);
+      fputs("\n", tool_stderr);
+      ptr += cut + 1; /* skip the space too */
+      len -= cut + 1;
+    }
+    else {
+      fputs(ptr, tool_stderr);
+      fputs("\n", tool_stderr);
+      len = 0;
     }
-    curl_free(print_buffer);
   }
 }
 
@@ -92,59 +80,62 @@ static void voutf(struct GlobalConfig *config,
  * Emit 'note' formatted message on configured 'errors' stream, if verbose was
  * selected.
  */
-void notef(struct GlobalConfig *config, const char *fmt, ...)
+void notef(const char *fmt, ...)
 {
-  va_list ap;
-  va_start(ap, fmt);
-  if(config->tracetype)
-    voutf(config, NOTE_PREFIX, fmt, ap);
-  va_end(ap);
+  if(global->tracetype) {
+    va_list ap;
+    va_start(ap, fmt);
+    voutf(NOTE_PREFIX, fmt, ap);
+    va_end(ap);
+  }
 }
 
 /*
  * Emit warning formatted message on configured 'errors' stream unless
  * mute (--silent) was selected.
  */
-void warnf(struct GlobalConfig *config, const char *fmt, ...)
+void warnf(const char *fmt, ...)
 {
-  va_list ap;
-  va_start(ap, fmt);
-  voutf(config, WARN_PREFIX, fmt, ap);
-  va_end(ap);
+  if(!global->silent) {
+    va_list ap;
+    va_start(ap, fmt);
+    voutf(WARN_PREFIX, fmt, ap);
+    va_end(ap);
+  }
 }
 
 /*
  * Emit help formatted message on given stream. This is for errors with or
  * related to command line arguments.
  */
-void helpf(FILE *errors, const char *fmt, ...)
+void helpf(const char *fmt, ...)
 {
   if(fmt) {
     va_list ap;
     va_start(ap, fmt);
     DEBUGASSERT(!strchr(fmt, '\n'));
-    fputs("curl: ", errors); /* prefix it */
-    vfprintf(errors, fmt, ap);
+    fputs("curl: ", tool_stderr); /* prefix it */
+    curl_mvfprintf(tool_stderr, fmt, ap);
     va_end(ap);
-    fputs("\n", errors); /* newline it */
+    fputs("\n", tool_stderr); /* newline it */
   }
-  fprintf(errors, "curl: try 'curl --help' "
+  curl_mfprintf(tool_stderr, "curl: try 'curl --help' "
 #ifdef USE_MANUAL
-          "or 'curl --manual' "
+                             "or 'curl --manual' "
 #endif
-          "for more information\n");
+                             "for more information\n");
 }
 
 /*
  * Emit error message on error stream if not muted. When errors are not tied
  * to command line arguments, use helpf() for such errors.
  */
-void errorf(struct GlobalConfig *config, const char *fmt, ...)
+void errorf(const char *fmt, ...)
 {
-  if(!config->silent || config->showerror) {
+  if(!global->silent || global->showerror) {
     va_list ap;
     va_start(ap, fmt);
-    voutf(config, ERROR_PREFIX, fmt, ap);
+    voutf(ERROR_PREFIX, fmt, ap);
     va_end(ap);
   }
 }
